import json

from bson import ObjectId

from pymongo import MongoClient
from random import randrange
from datetime import datetime
import os
# from UtilsGit import UtilsGit
from pathlib import Path
import string
from xml.etree import ElementTree as ET
from xml.dom import minidom
from xml.etree.ElementTree import Element, SubElement, Comment

from werkzeug.utils import secure_filename

from config.gitUtils import GitUtils
from config.utils import Utils
from modele.encours_dao import EncoursDao


class ContenuDao:
    client = MongoClient('localhost', 27017)
    chemin = "/home/OpenQuizz/contenus/"
    gitUtils = GitUtils()

    ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
    base_donnees = client.squelette

    ## Fonction permettant d'ajouter un contenu dans le panier de l'auteur
    # @param l'auteur en cours
    # @param nom_fichier le nom du fichier associé au contenu
    # @param commit le 'hash' du commit associé (récupéré via git)
    def ajouter_contenu(self, auteur, contenu_a_ecrire, titre, tags):

        # créer la métadonnée dans la base mongo
        self.base_donnees.contenu.insert_one(
            {"auteur": auteur, "popularite": 0,
             "fichier": "", "titre": titre,
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "note": 0, "nbreNotes": 0, "commit": ""},
             "tags_fixes": tags,
             "type": " ", "source": "markdown"})

        nom_contenu = self.id_dernier_contenu_ajoute()

        chemin_tags = '/'.join(tags) + "/"
        chemin_fichier = self.chemin + chemin_tags + nom_contenu + "/"
        Path(chemin_fichier).mkdir(0o777, parents=True, exist_ok=True)

        myFile = open(chemin_fichier + nom_contenu, "w")
        # y ajouter le texte du contenu
        myFile.write(contenu_a_ecrire)
        myFile.close()

        # mettre a jour le chemin du contenu
        self.base_donnees.contenu.update_one({"_id": ObjectId(nom_contenu)},
                                             {"$set": {"fichier": chemin_tags + nom_contenu + "/" + nom_contenu}})

        # initialiser le git et récupérer le commit
        commit = self.gitUtils.commit(chemin_fichier, nom_contenu, "version" + str(1))

        #  puis le mettre a jour dans le contenu
        self.base_donnees.contenu.update_one({"_id": ObjectId(nom_contenu)},
                                             {"$set": {"versionEnCours.commit": commit}})

    ## Fonction qui permet d'importer un nouveau contenu
    # @param auteur l'auteur qui est en train d'importer le contenu
    # @param file le fichier importe
    # @param tags les tags associes au fichier importe
    # @param descriptif (optionnel). Si le fichier n'est pas un xml, doit être renseigné
    # @param type (optionnel). Si le fichier n'est pas un xml, indique le type du fichier importé
    # @param titreFichier (optionnel). Si le fichier n'est pas un xml, indique le titre du fichier importé
    def importer_contenu(self, auteur, file, tags, descriptif=None, type=None, titreFichier=None):

        from xml.dom import minidom

        global chemin_fichier, filename, chemin_tags, titre
        if file:
            filename = secure_filename(file.filename)

            chemin_tags = '/'.join(json.loads(tags)) + "/"



            chemin_fichier = self.chemin + chemin_tags + filename + "/"
            Path(chemin_fichier).mkdir(0o777, parents=True, exist_ok=True)
            file.save(os.path.join(chemin_fichier, filename))


            if titreFichier is None:
                # on récupère le vrai titre du contenu
                xmldoc = minidom.parse(chemin_fichier + filename)
                mite = xmldoc.getElementsByTagName('name')
                titre = mite[0].childNodes[1].firstChild.nodeValue

            else:
                titre = titreFichier

        # créer la métadonnée dans la base mongo
        self.base_donnees.contenu.insert_one(
            {"auteur": auteur, "popularite": 0,
             "fichier": chemin_tags + filename + "/" + filename, "titre": titre,
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "note": 0, "nbreNotes": 0, "commit": ""},
             "tags_fixes": tags,
             "type": "quiz", "source": "moodle"})

        nom_contenu = self.id_dernier_contenu_ajoute()

        # initialiser le git et récupérer le commit
        commit = self.gitUtils.commit(chemin_fichier, filename, "version" + str(1))

        #  puis le mettre a jour dans le contenu
        self.base_donnees.contenu.update_one({"_id": ObjectId(nom_contenu)},
                                             {"$set": {"versionEnCours.commit": commit}})

        # si le contenu est chargé avec un descriptif, on l'insere
        if descriptif is not None:
            self.base_donnees.contenu.update_one({"_id": ObjectId(nom_contenu)},
                                                 {"$set": {"source": 'autre', "type": type,
                                                           "descriptif": descriptif}})

    ## récupérer la popularité d'une version
    # @param id_contenu l'id du contenu
    # @param version le numéro de version
    def get_popularite(self, id_contenu):
        return self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["popularite"]

    ## incrémenter la popularité d'un contenu ou d'une de ses versions
    # @param id_contenu l'id du contenu
    # @param version le numéro de version incrémenté
    def incrementer_popularite(self, id_contenu):

        popularite = self.get_popularite(id_contenu)

        self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)},
                                             {"$set": {"popularite": popularite + 1}})

    ## recherche d'un contenu par id (pour affichage de son contenu. charge la version courante
    # @param id_contenu l'id du contenu recherché
    def get_contenu_par_id(self, id_contenu):
        return self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})

    ## Récupérer l'id du dernier contenu ajouté
    # @return l'id du dernier contenu (sous la forme d'une chaine de caracteres)
    def id_dernier_contenu_ajoute(self):
        # créer un fichier (au bon endroit)
        # récupérer l'id mongo (pour nommer le répertoire)
        tmp = self.base_donnees.contenu.find().sort("_id", -1).limit(1)

        record = next(tmp, None)
        return str(record.get("_id"))

    ## recherche des contenus pour un auteur donné (son 'panier')
    # @param auteur l'auteur des contenus
    # @param perso true si contenus persos a charger, false sinon
    # @param tiers true si contenus tiers a charger, false sinon
    # @param en_vours true si contenus en cours a charger, false sinon
    # @return tous les contenus présents dans le panier de l'auteur
    def get_contenus(self, auteur, perso, tiers, en_cours):

        contenus = list()

        if tiers:
            id_tiers = self.get_contenus_tiers(auteur)
            for i in range(len(id_tiers)):
                contenu_tiers = self.get_contenu_par_id(id_tiers[i][0]["contenu"])
                contenu_tiers["tags_libres"] = self.recuperer_tags_libres_contenu(id_tiers[i][0]["contenu"], auteur)
                contenu_tiers["provenance"] = "tiers"
                contenu_tiers["version"] = id_tiers[i][1]["version"]
                note = self.recupererNoteContenu(id_tiers[i][0]["contenu"], id_tiers[i][1]["version"], auteur)
                if note == -1:
                    contenu_tiers["note_perso"] = "-"
                else:
                    contenu_tiers["note_perso"] = note
                contenus.append(contenu_tiers)

        if perso:
            curseur_perso = self.get_contenus_persos(auteur)
            for contenu in curseur_perso:
                contenu["provenance"] = "perso"
                contenu["tags_libres"] = self.recuperer_tags_libres_contenu(contenu["_id"], auteur)
                contenus.append(contenu)

        if en_cours:
            curseur_en_cours = EncoursDao().get_contenus_en_cours_auteur(auteur)
            for contenu in curseur_en_cours:
                contenu["provenance"] = "en-cours"
                contenu["tags_libres"] = self.recuperer_tags_libres_contenu(contenu["_id"], auteur)
                contenus.append(contenu)
            # on garde le développement de l'évaliation pour plus tard
            # return contenus_persos + contenus_en_cours + contenus_tiers
        return contenus

    ## FOnction permettant de récupérer les contenus tiers d'un auteur
    # @param auteur l'auteur en question
    # return les contenus tiers de l'auteur (_id et version)
    def get_contenus_tiers(self, auteur):
        return self.base_donnees.utilisateur.find_one({"nom": auteur})["tiers"]

    ## Fonction permettant de charger tous les contenus 'publiés' n'appartenant pas à auteur (pour afficher sa banque)
    # @param auteur l'auteur en cours
    # @return tous les contenus non présents dans le panier de l'auteur
    def get_contenus_non_auteur(self, auteur):
        contenus_tiers = self.get_contenus_tiers(auteur)
        tiers_auteur = list()
        for tier in contenus_tiers:
            tiers_auteur.append(ObjectId(tier[0]["contenu"]))

        return self.base_donnees.contenu.find({"$and": [{"auteur": {"$ne": auteur}}, {"_id": {"$nin": tiers_auteur}}]})

    ## FOnction de création d'un tiers
    # @param auteur l'auteur en cours
    # @param id_contenu le contenu à ajouter en tiers
    # @param version la version dans laquelle le contenu a été pris
    def creer_tiers(self, auteur, id_contenu, version):
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {
            "$push": {
                "tiers": [{"contenu": ObjectId(id_contenu)}, {"version": version}]}})
        self.incrementer_popularite(id_contenu)

    ## FOnction permettant de récupérer les contenus persos d'un auteur
    # @param auteur l'auteur en question
    # return tous les contenus personnels de l'auteur
    def get_contenus_persos(self, auteur):
        return self.base_donnees.contenu.find({"auteur": auteur})

    ## l'auteur 'renie' son contenu, qui reste disponible dans le panier de ceux qui le possédent
    # @param auteur l'id du contenu à renier
    def dissocier_contenu(self, id_contenu):
        self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)}, {"$set": {"auteur": "Aucun"}})

    ## permet de, soit renier, soit supprimer définitivement un auteur (et le fichier de contenu associé)
    # @param id_contenu l'id du contenu à supprimer
    def supprimer_contenu(self, id_contenu):
        import shutil

        if self.existe_en_tiers(id_contenu) != 0:
            self.dissocier_contenu(id_contenu)

        else:
            chemin = self.get_chemin_du_contenu(id_contenu)
            repertoire = chemin[:chemin.rfind("/")]
            shutil.rmtree(repertoire)
            self.base_donnees.contenu.delete_one({"_id": ObjectId(id_contenu)})

    ## suppression défintive d'un tiers (de la référence)
    # @param id_tiers l'id du tiers à supprimer
    # @param auteur l'auteur concerné
    # @param version la version du tiers à supprimer
    # @param la note associee au contenu
    def supprimer_tiers(self, id_tiers, auteur, version, note):

        note_contenu = self.recupererNoteContenu(id_tiers, auteur, version)
        # on distingue le cas ou une note a été donnée ou non
        if note == "-":
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$pull": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": version}]}})

        else:
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$pull": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": version},
                                                                   {"note": int(note)}]
                                                     }})

        # si le contenu n'est plus possédé par personne, on le supprime définitivement
        if self.is_anonyme(id_tiers) and self.existe_en_tiers(id_tiers) == 0:
            self.supprimer_contenu(id_tiers)

    ## fonction qui permet de savoir le nombre de fois qu'un contenu est présent en 'contenu tiers' chez d'autres auteurs
    # @param auteur l'id du contenu à 'tester'
    # @return le nombre de contenus tiers associés
    def existe_en_tiers(self, id_contenu):

        return self.base_donnees.utilisateur.count_documents(
            {"tiers": {"$elemMatch": {"$elemMatch": {"contenu": ObjectId(id_contenu)}}}})

    ## fonction qui permet de créer un contenu à partir d'une contenu en cours
    # @param chemin le chemin du contenu
    # @param auteur_en_cours l'auteur du contenu en cours
    # @param id_contenu_en_cours l'id du contenu en cours
    def creer_nouveau_contenu(self, id_demande_publication, auteur_en_cours):
        from modele.demande_pubDao import DemandePubDao

        # on récupère l'id du contenu en cours associés à la demande de publication
        id_contenu_en_cours = DemandePubDao().get_id_contenu_en_cours_associe_a_une_publication(
            id_demande_publication)

        # ce dernier permettant de récupérer le texte à écrire dans le contenu nouvellement créé
        contenu_a_ecrire = Utils().recuperer_contenu_fichier(id_contenu_en_cours, True)

        # on utilise la fonction déja existante pour créer le contenu
        self.ajouter_contenu(auteur_en_cours, contenu_a_ecrire, "", [])

        # et on détruit la demande de publication et le contenu en cours associé, pour finir
        DemandePubDao().supprimer_publication(id_demande_publication)
        EncoursDao().supprimer_contenu_en_cours_via_id(id_contenu_en_cours)

    ## récupère le chemin d'un contenu
    # @param id_contenu l'id du contenu recherché
    # @return le chemin désiré
    def get_chemin_du_contenu(self, id_contenu):

        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)

        return self.chemin + self.base_donnees.contenu.find_one({"_id": id_contenu})["fichier"]

    ## récupère la version en cours du contenu
    # @param id_contenu l'id du contenu
    # @return la version courante du contenu
    def get_version_courante(self, id_contenu):
        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)
        return self.base_donnees.contenu.find_one({"_id": id_contenu})["versionEnCours"]["numero"]

    ## récupère la version d'un contenu mis en tiers (ou 0 si ce contenu n'est pas sossédé en tiers)
    # @param auteur l'auteur concerné
    # @param id_contenu le contenu recherché
    # @return la version du tiers id_contenu (ou 0)
    def get_version_tiers_avec_id(self, id_contenu, auteur):
        tiers = (self.base_donnees.utilisateur.find_one({"nom": auteur})["tiers"])
        for i in range(len(tiers)):
            tmp = tiers[i]
            if tmp[0]["contenu"] == ObjectId(id_contenu):
                return tmp[1]["version"]
        return 0

    ## mise à jour d'un contenu
    # @param idContenu l'id du contenu à mettre à jour
    # @param nomBranche le nom de la branche
    # @param auteur l'auteur du contenu à mettre à jour
    # @param enCours permet de déterminer si l'origine de la maj est un contenu en cours ou une demande de publication

    def maj_contenu(self, id_contenu_en_cours, auteur=None, enCours=True):
        from modele.demande_pubDao import DemandePubDao

        if (enCours):

            id_contenu = EncoursDao().get_id_origine_contenu_en_cours(id_contenu_en_cours)

        else:

            id_contenu = DemandePubDao().get_id_contenu_associe_a_une_publication(id_contenu_en_cours)
            id_contenu_en_cours = DemandePubDao().get_id_contenu_en_cours_associe_a_une_publication(id_contenu_en_cours)

        chemin = self.get_chemin_du_contenu(id_contenu)

        repertoire = chemin[:chemin.rfind("/") + 1]
        version = int(self.get_version_courante(id_contenu)) + 1

        # Mise à jour du répertoire git
        # idCommit = UtilsGit.mergeVersion(repertoire, nomBranche)

        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)

        # Mise à jour des métadonnées - ok
        version_en_cours = self.base_donnees.contenu.find_one({"_id": id_contenu})["versionEnCours"]
        self.base_donnees.contenu.update_one({"_id": id_contenu}, {"$push": {"historique": version_en_cours}})
        self.base_donnees.contenu.update_one({"_id": id_contenu}, {
            "$set": {"versionEnCours": {"numero": version, "date": datetime.now().strftime("%d/%m/%y"), "note": 0,
                                        "nbreNotes": 0,
                                        "commit": ""}}})

        # Si un auteur est passé en paramètres de la fonction maj = on l'ajoute dans les métadonnées
        if auteur is not None:
            self.base_donnees.contenu.update_one({"_id": id_contenu}, {
                "$set": {"auteur": auteur}})
            note = self.recupererNoteContenu(id_contenu, version - 1, auteur)
            self.supprimer_tiers(id_contenu, auteur, version - 1, note)

        # on modifie le texte du contenu d'origine
        nouveau_texte = Utils().recuperer_contenu_fichier(id_contenu_en_cours, True)

        titre_en_cours = EncoursDao().get_titre_en_cours(id_contenu_en_cours)


        Utils().changer_texte_fichier(id_contenu, nouveau_texte, titre_en_cours, True)

        ## on supprime le contenu en cours
        EncoursDao().supprimer_contenu_en_cours_via_id(id_contenu_en_cours)

        # initialiser le git et récupérer le commit
        version = self.get_version_courante(id_contenu)
        index = chemin.rindex("/")
        nom = chemin[index + 1:]

        chemin = chemin[:index]

        commit = self.gitUtils.commit(chemin, nom, "version" + str(version + 1))

        #  puis le mettre a jour dans le contenu
        self.base_donnees.contenu.update_one({"_id": id_contenu}, {"$set": {"versionEnCours.commit": commit}})

    ## lecture d'un contenu (version basique pour les tests)
    # @param id l'id du contenu à consulter
    # @param auteur l'auteur souhaitant consulter le contenu
    def consulter_un_contenu(self, id, auteur):
        # Créer un répertoire au nom de l'auteur (ou l'utiliser si il existe déja)
        auteur = auteur.lower().replace(" ", "_")
        repertoire_auteur = self.chemin + auteur + "/"
        Path(repertoire_auteur).mkdir(0o755, parents=True, exist_ok=True)
        # Copier le fichier lié au contenu dans ce répertoire
        chemin = self.get_chemin_du_contenu(id)
        nom_fichier = chemin[chemin.rfind("/") + 1:]
        os.system("cp '" + chemin + "'  '" + repertoire_auteur + nom_fichier + "'")
        # self.affichage(chemin)

    ## après fin de consultation, destruction du fichier temporaire généré
    # @param id l'id du contenu consulté
    # @param auteur l'auteur auant effectué la consultation
    def detruire_fichier_tmp(self, id, auteur):
        auteur = auteur.lower().replace(" ", "_") + "/"
        chemin = self.get_chemin_du_contenu(id)
        nom_fichier = chemin[chemin.rfind("/") + 1:]
        os.system("rm '" + self.chemin + auteur + nom_fichier + "'")

    ## Fonction permettant de déterminer si un contenu est "anonyme"
    # @param idCOntenu le contenu à tester
    # @return booléen true.false si le contenu est anonyme ou pas
    def is_anonyme(self, idContenu):
        return self.base_donnees.contenu.find_one({"_id": ObjectId(idContenu)})["auteur"] == "Aucun"

    ## Récupérer tous les contenus
    # @return tous les contenus
    def get_tous_les_contenus(self):
        return self.base_donnees.contenu.find()

    # FOnction permettant de déterminer si un certain contenu fait partie des tiers d'un auteur
    # @param auteur l'auteur concerné
    # @param idContenu le contenu à 'tester'
    # @return un booléen correspondant à savoir si le contenu est chez l'auteur ou pas
    def is_tiers_auteur(self, auteur, idContenu):
        return self.get_version_tiers_avec_id(ObjectId(idContenu), auteur) != 0

    ## mettre à jour un contenu 'tiers'
    # @param auteur l'auteur concerné
    # @param id_favori le favori à maj
    # @param nouvelle_version le numero de la nouvelle version
    # @param refuse indique si la mise a jour est acceptee ou refusee
    def maj_tiers(self, id_tiers, nouvelle_version, auteur, refuse=False):
        if not refuse:

            # on vire le précédent
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$pull": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": nouvelle_version - 1}]}})

            # et on ajoute la version maj
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$push": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": nouvelle_version}]}})
        else:
            # on vire le précédent
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$pull": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": nouvelle_version - 1}]}})

            # et on ajoute la version maj
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$push": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": str(nouvelle_version - 1) + 'r'}]}})

    ## Permet de savoir si au moins un tiers est mettable à jour pour un auteur
    # @param auteur l'auteur concerné
    # @return un booléen indiquant si on a des mises à jour faisables
    def is_tiers_maj(self, auteur):
        # on cherche toutes les versions en cours dans les tiers et on compare avec celles de la banque

        # On réfcupère tous les ties d'un auteur
        tiers = self.get_contenus_tiers(auteur)
        for i in range(len(tiers)):
            versionCourante = self.get_version_courante(tiers[i][0]["contenu"])
            if tiers[i][1]["version"] < versionCourante:
                return True

        return False

    ## Permet de savoir si au moins un tiers est mettable à jour pour un auteur
    # @param auteur l'auteur concerné
    # @return la liste des tiers mettables à jour
    def liste_tiers_maj(self, auteur):
        liste = list()
        # On réfcupère tous les ties d'un auteur
        tiers = self.get_contenus_tiers(auteur)
        for i in range(len(tiers)):
            versionCourante = self.get_version_courante(tiers[i][0]["contenu"])
            if tiers[i][1]["version"] < versionCourante:
                liste.append(self.get_contenu_par_id(tiers[i][0]["contenu"]))

        return liste

    ## Fonction permettant de récupérer le 'hash' d'un commit
    # @param id_contenu l'id du contenu concerné
    # @param version la version pour laquelle on veux récupérer le commit
    # @return le commit associé à la version
    def get_commit_pour_une_version(self, id_contenu, version):

        version = str(version)
        if version == str(self.get_version_courante(id_contenu)) or version == '0':
            courante = self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["versionEnCours"]
            return courante["commit"]
        else:
            historique = self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["historique"]
            for elt in historique:
                if str(elt["numero"]) == version:
                    return elt["commit"]

    ## Fonction permettant de récupérer les tags associés à un contenu
    # @param id_contenu l'id du contenu concerné
    # @return les tags associés au contenu
    def get_tags_contenu(self, id_contenu):
        return self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["tags_fixes"]

    ## Mise à jour des tags fixes associés au contenu
    # @param id_contenu l'id du contenu concerné
    # @param tags les nouveaux tags
    def changer_tags_contenus(self, id_contenu, tags):

        ancien_chemin = self.get_chemin_du_contenu(id_contenu)

        # on modifie les tags
        self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)}, {"$set": {"tags_fixes": tags}})
        # et le chemin du fichier
        chemin_tags = '/'.join(tags) + "/"

        # mettre a jour le chemin du contenu
        self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)},
                                             {"$set": {"fichier": chemin_tags + id_contenu + "/" + id_contenu}})

        Path(self.chemin + chemin_tags + id_contenu + "/").mkdir(0o777, parents=True, exist_ok=True)


        Path(ancien_chemin).rename(self.get_chemin_du_contenu(id_contenu))
        self.rm_tree(Path(ancien_chemin[:ancien_chemin.rfind("/") + 1]))
        # Path(ancien_chemin[:ancien_chemin.rfind("/") + 1]).rmdir()

    ## Fonction utilitaire -> supprimer un repertoire
    # @param path le chemin du repertoire
    def rm_tree(self, pth):
        for child in pth.iterdir():
            if child.is_file():
                child.unlink()
            else:
                self.rm_tree(child)
        pth.rmdir()

    ## Modification d'un titre d'un contenu personnel
    # @param id_en_cours l'id du contenu
    # @param nouveau_titre le titre à changer
    def changer_titre(self, id_en_cours, nouveau_titre):
        self.base_donnees.contenu.update_one({"_id": ObjectId(id_en_cours)}, {"$set": {"titre": nouveau_titre}})

    ## Fonction permettant de créer un tag libre et de l'associer ou non à un contenu
    # @param auteur l'auteur du nouveau tag
    # @param tag le tag créé
    # @param id_contenu l'id du contenu associé (potentiellement vide)
    def creer_tag_libre(self, auteur, tag):

        self.base_donnees.utilisateur.update_one({"nom": auteur}, {
            "$push": {
                "tags": [{"tag": tag}, {"contenus": []}]}})

    ## Fonction permettant de supprimer un tag libre
    # @param auteur l'auteur du nouveau tag
    # @param tag le tag a supprimer
    # @param id_contenu l'id du contenu associé (potentiellement vide)
    def supprimer_tag_libre(self, auteur, mon_tag):
        # on récupère les tags
        tags = self.base_donnees.utilisateur.find_one({"nom": auteur})["tags"]
        # on "vide" les tags (qu'on réinjectera)
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": []}})

        for tag in tags:
            if tag[0]['tag'] == mon_tag:
                tags.remove(tag)

        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": tags}})

    ## Fonction permettant de dissocier un contenu d'un tag libre
    # @param auteur l'auteur du nouveau tag
    # @param tag le tag concerné
    # @param id_contenu l'id du contenu à dissocier
    def dissocier_tag_libre(self, auteur, mon_tag, id_contenu):
        # on récupère les tags
        tags = self.base_donnees.utilisateur.find_one({"nom": auteur})["tags"]
        # on "vide" les tags (qu'on réinjectera)
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": []}})

        # on parcourt chaque tag et on vire les mentions à id_contenu
        for tag in tags:
            if tag[0]['tag'] == mon_tag:
                tag[1]['contenus'].remove(ObjectId(id_contenu))
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": tags}})

    ## Fonction permettant d'associer un contenu à un tag libre
    # @param auteur l'auteur du nouveau tag
    # @param tag le tag concerné
    # @param id_contenu l'id du contenu à associer au tag
    def associer_tag_libre(self, auteur, tag, id_contenu):

        # on récupère les tags
        tags = self.base_donnees.utilisateur.find_one({"nom": auteur})["tags"]
        # on "vide" les tags (qu'on réinjectera)
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": []}})
        # on parcourt chaque tag et on vire les mentions à id_contenu
        for letag in tags:
            if letag[0]['tag'] == tag:
                letag[1]['contenus'].append(ObjectId(id_contenu))
                letag[1]['contenus'] = list(set(letag[1]['contenus']))
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": tags}})

    ## Fonction permettant de récupérer tous les tags libres d'un auteur
    # @param auteur l'auteur du nouveau tag
    # @return les tags libres de l'auteur
    def recuperer_all_tags_libres(self, auteur):
        tags = self.base_donnees.utilisateur.find_one({"nom": auteur})["tags"]
        tags_auteur = list()
        for f in tags:
            tags_auteur.append(f[0]['tag'])
        return tags_auteur

    # recuperer_tags_libres_contenu
    ## Fonction permettant de récupérer les tags libres d'un auteur associés à un contenu en particulier
    # @param auteur l'auteur du nouveau tag
    # @param id_contenu le contenu associé
    # @return les tags libres de l'auteur
    def recuperer_tags_libres_contenu(self, id_contenu, auteur):
        tags = self.base_donnees.utilisateur.find_one({"nom": auteur})["tags"]
        tags_auteur = list()
        for f in tags:
            for cont in f[1]['contenus']:
                if cont == ObjectId(id_contenu):
                    tags_auteur.append(f[0]['tag'])
        return tags_auteur

    ## Fonction permettant de noter un contenu (pour une version donnée)
    # @param id_contenu l'id du contenu concerné
    # @param version la version du contenu notée
    # @param auteur l'auteur qui met la note
    # @param note la note affectée au contenu
    def noterContenuTiers(self, id_contenu, version, auteur, note):

        # on vire le précédent
        self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                 {"$pull": {
                                                     "tiers": [{"contenu": ObjectId(id_contenu)},
                                                               {"version": version}]}})

        # et on ajoute la version maj
        self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                 {"$push": {
                                                     "tiers": [{"contenu": ObjectId(id_contenu)},
                                                               {"version": version},
                                                               {"note": note}]}})

        # et on modifie le contenu de base (sa moyenne)
        return self.modifierMoyenneContenu(id_contenu, version, note)

    ## Fonction permettant de récupérer une note donnée
    # @param id_contenu l'id du contenu concerné
    # @param version la version du contenu notée
    # @param auteur l'auteur qui met la note
    def recupererNoteContenu(self, id_contenu, version, auteur):

        if type(id_contenu) == str:
            id_contenu = ObjectId(id_contenu)

        if type(version) == str:
            version = int(version)

        contenus_tiers = self.base_donnees.utilisateur.find_one({"nom": auteur})["tiers"]
        for contenu in contenus_tiers:

            if len(contenu) == 3 and contenu[0]["contenu"] == id_contenu and contenu[1]["version"] == version:
                return json.dumps(contenu[2]["note"])

        return '-'

    ## Fonction permettant de récupérer une note globale pour une version d'un contenu
    # @param id_contenu l'id du contenu concerné
    # @param version la version du contenu notée
    def recupererNoteGlobaleContenu(self, id_contenu, version):
        if int(version) == self.get_version_courante(id_contenu):
            return self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["versionEnCours"]["note"] \
                   / self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["versionEnCours"]["nbreNotes"]
        else:
            historique = self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["historique"]
            # todo PARTIE MANQUANTE - A DEVELOPPER
            # DANS CETTE PARTIE ON DOIT ETRE CAPABLE D'ALLER CHERCHER LA NOTE D'UNE VERSION QUI N'EST PAS LA VERSION EN COURS
            return "-"

    ## Fonction permettant de changer la moyenne d'une version d'un contenu à l'insertion d'une nouvelle note
    # @param id_contenu l'id du contenu concerné
    # @param version la version du contenu notée
    # @param note la note affectée au contenu
    def modifierMoyenneContenu(self, id_contenu, version, note):
        if version == self.get_version_courante(id_contenu):
            versionEnCours = self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["versionEnCours"]
            self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)}, {"$set": {"versionEnCours": []}})
            versionEnCours['note'] = versionEnCours['note'] + note
            versionEnCours['nbreNotes'] = versionEnCours['nbreNotes'] + 1
            self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)},
                                                 {"$set": {"versionEnCours": versionEnCours}})

            return json.dumps([versionEnCours['note'], versionEnCours['nbreNotes']])


        else:
            contenus = self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["historique"]
            self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)}, {"$set": {"historique": []}})
            for cont in contenus:
                if cont['numero'] == version:
                    cont['note'] = cont['note'] + note
                    cont['nbreNotes'] = cont['nbreNotes'] + 1
                    self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)},
                                                         {"$set": {"historique": contenus}})
                    return json.dumps([cont['note'], cont['nbreNotes']])

    # # Fonction qui permet de parser un contenu (de type XML - Moodle) pour une version donnée
    def previsualiser_xml_moodle_version(self, id_contenu, version, en_cours=False):
        Utils().changer_version(id_contenu, version)
        return self.previsualiser_xml_moodle(id_contenu, en_cours)

    # # Fonction qui permet de parser un contenu (de type XML - Moodle) -Fonction utile pour la prévisualisation &
    # l'édition @param id_contenu l'id du contenu à parser @return un dictionnaire comprenant toutes les informations
    # utiles
    def previsualiser_xml_moodle(self, id_contenu, en_cours=False):

        texte = Utils().recuperer_contenu_fichier(id_contenu, en_cours)
        # on va stocker tous les éléments séparés dans cette structure
        elements_question = dict()

        root = ET.fromstring(texte)

        for question in root.iter('question'):

            if question.attrib["type"] != "category":
                elements_question["type_question"] = question.attrib["type"]

        for note in root.iter('defaultgrade'):
            elements_question["note_defaut"] = note.text

        for penalty in root.iter('penalty'):
            elements_question["penalite"] = penalty.text

        for name in root.iter('name'):
            for nom in name.iter('text'):
                elements_question["nom_question"] = nom.text

        for question in root.iter('questiontext'):
            for intitule in question.iter('text'):
                elements_question["intitule_question"] = intitule.text

        for general in root.iter('generalfeedback'):
            for feedback in general.iter('text'):
                elements_question["feedback_general"] = feedback.text

        if elements_question["type_question"] == "truefalse":
            note_faux = 0

            # on récupère la bonne réponse (mise par défault à Vrai mais changée si besoin).
            elements_question["bonne_reponse"] = 'Vrai'
            for answer in root.iter('answer'):
                note_faux = answer.attrib["fraction"]
            # On analyse la dernière réponse enrengistrée dans le for. Si elle est égale à 100, on change la bonne réponse.
            if note_faux == '100':
                elements_question["bonne_reponse"] = 'Faux'

            # on récupère le texte associé aux deux réponses (pas forcément vrai / faux, l'auteur peut mettre a peu près ce qu'il veut)
            liste_feedback = []
            for answer in root.iter('answer'):
                for feedback in answer.iter('text'):
                    liste_feedback.append(feedback.text)

            elements_question["liste_feedback"] = liste_feedback

        elif elements_question["type_question"] == "shortanswer":
            # majuscule -> permet la casse (à 1) ou non (à 0)
            for usecase in root.iter('usecase'):
                elements_question["majuscule"] = usecase.text
            # on récupère les réponses valables, les notes associées et les feedback
            liste_reponse = []
            liste_note = []
            liste_feedback = []
            for answer in root.iter('answer'):
                liste_note.append(answer.attrib["fraction"])
                for question in answer.iter('text'):
                    liste_reponse.append(question.text)
                for feedback in answer.iter('feedback'):
                    for feed in feedback.iter('text'):
                        liste_feedback.append(feed.text)


            elements_question["liste_reponse"] = liste_reponse
            elements_question["liste_note"] = liste_note
            elements_question["liste_feedback"] = liste_feedback

        elif elements_question["type_question"] == "multichoice":

            for single in root.iter('single'):
                elements_question["nb_reponse"] = single.text

            for shuffleanswers in root.iter('shuffleanswers'):
                elements_question["melange_reponse"] = shuffleanswers.text

            for answernumbering in root.iter('answernumbering'):
                elements_question["numerotation"] = answernumbering.text

            liste_reponse = []
            liste_note = []
            liste_feedback = []
            for answer in root.iter('answer'):
                liste_note.append(answer.attrib["fraction"])
                for question in answer.iter('text'):
                    if question.text is not None:
                        liste_reponse.append(question.text)
                for feedback in answer.iter('feedback'):
                    for feed in feedback.iter('text'):
                        liste_feedback.append(feed.text)

            elements_question["liste_reponse"] = liste_reponse
            elements_question["liste_note"] = liste_note
            elements_question["liste_feedback"] = liste_feedback

            liste_feedback_principaux = []
            for feedback in root.iter('text'):
                liste_feedback_principaux.append(feedback.text)

            elements_question["liste_feedback_principaux"] = liste_feedback_principaux

        elif elements_question["type_question"] == "numerical" or \
                elements_question["type_question"] == "calculatedsimple":

            for unit in root.iter('unitgradingtype'):
                elements_question["traitement_unite"] = unit.text

            for showunits in root.iter('showunits'):
                elements_question["saisie_unite"] = showunits.text

            for unitsleft in root.iter('unitsleft'):
                elements_question["position_unite"] = unitsleft.text

            liste_unite_nom = []
            for unit_name in root.iter('unit_name'):
                liste_unite_nom.append(unit_name.text)
            for k in range(3):
                liste_unite_nom.append('')

            liste_unite_coef = []
            for multiplier in root.iter('multiplier'):
                liste_unite_coef.append(multiplier.text)
            for k in range(3):
                liste_unite_coef.append('')

            elements_question["liste_unite_nom"] = liste_unite_nom
            elements_question["liste_unite_coef"] = liste_unite_coef

            liste_reponse = []
            liste_note = []
            liste_feedback = []
            liste_tolerance = []
            liste_tolerance_type = []
            liste_reponse_format = []
            liste_reponse_taille = []
            for answer in root.iter('answer'):
                liste_note.append(answer.attrib["fraction"])
                for question in answer.iter('text'):
                    if question.text is not None:
                        liste_reponse.append(question.text)
                for feedback in answer.iter('feedback'):
                    for feed in feedback.iter('text'):
                        liste_feedback.append(feed.text)
                for tolerance in answer.iter('tolerance'):
                    if tolerance.text is not None:
                        liste_tolerance.append(tolerance.text)
                if elements_question["type_question"] == "calculatedsimple":
                    for type in answer.iter('tolerancetype'):
                        if type.text is not None:
                            liste_tolerance_type.append(type.text)
                    for format in answer.iter('correctanswerformat'):
                        if format.text is not None:
                            liste_reponse_format.append(format.text)
                    for length in answer.iter('correctanswerlength'):
                        if length.text is not None:
                            liste_reponse_taille.append(length.text)

            elements_question["liste_reponse"] = liste_reponse
            elements_question["liste_note"] = liste_note
            elements_question["liste_feedback"] = liste_feedback
            elements_question["liste_tolerance"] = liste_tolerance
            if elements_question["type_question"] == "calculatedsimple":
                elements_question["liste_tolerance_type"] = liste_tolerance_type
                elements_question["liste_reponse_format"] = liste_reponse_format
                elements_question["liste_reponse_taille"] = liste_reponse_taille

        return elements_question

    ## Fonction permettant de modifier un xml Moodle
    # @param id_contenu
    # @param elements_contenu
    def modifier_xml_moodle(self, id_contenu, elements_contenu, en_cours=False):
        # On charge le fichier actuel
        texte = Utils().recuperer_contenu_fichier(id_contenu, en_cours)
        root = ET.fromstring(texte)


        # On parcourt tous les champs généralistes (type_question est inamovible)
        for note in root.iter('defaultgrade'):
            note.text = elements_contenu["note_defaut"]

        for penalty in root.iter('penalty'):
            penalty.text = elements_contenu["penalite"]

        for name in root.iter('name'):
            for nom in name.iter('text'):
                nom.text = elements_contenu["nom_question"]

        for question in root.iter('questiontext'):
            for intitule in question.iter('text'):
                intitule.text = elements_contenu["intitule_question"]

        for general in root.iter('generalfeedback'):
            for feedback in general.iter('text'):
                feedback.text = elements_contenu["feedback_general"]

            # puis on fait la même chose que dans la fonction d'au dessus, on parcourt les éléments spécifiques en fonction du type_question

        if elements_contenu["type_question"] == "truefalse":
            note_faux = 0
            # # on récupère la bonne réponse (mise par défault à Vrai mais changée si besoin).
            i = 0
            for answer in root.iter('answer'):
                if i == 0:
                    answer.attrib["fraction"] = '100' if elements_contenu['bonne_reponse'] == 'Vrai' else '0'
                elif i == 1:
                    answer.attrib["fraction"] = '100' if elements_contenu['bonne_reponse'] == 'Faux' else '0'
                i = i + 1

        elif elements_contenu["type_question"] == "shortanswer":
            pass
            # majuscule -> permet la casse (à 1) ou non (à 0)
            for usecase in root.iter('usecase'):
                usecase.text = elements_contenu["majuscule"]

            # on récupère les réponses valables, les notes associées et les feedback

            i = 0

            for answer in root.iter('answer'):
                answer.attrib["fraction"] = elements_contenu["liste_note"][i]
                for question in answer.iter('text'):
                    if question.text is not None:
                        question.text = elements_contenu["liste_reponse"][i]
                for feedback in answer.iter('feedback'):
                    for feed in feedback.iter('text'):
                        feed.text = elements_contenu["liste_feedback"][i]

                i += 1

            if i < len(elements_contenu["liste_reponse"]):
                for question in root.iter('question'):
                    # eltQuestion = root.iter("question")
                    new_answer = SubElement(question, "answer")
                    new_answer.attrib["fraction"] = elements_contenu["liste_note"][i]
                    new_answer.attrib["format"] = "moodle_auto_format"
                    text = SubElement(new_answer, "text")
                    text.text = elements_contenu["liste_reponse"][i]
                    feedback = SubElement(new_answer, "feedback")
                    text_feedback = SubElement(feedback, "text")
                    text_feedback.text = elements_contenu["liste_feedback"][i]
                    i += 1

        elif elements_contenu["type_question"] == "multichoice":
            pass

            for single in root.iter('single'):
                single.text = elements_contenu["multipleChoisie"]

            for shuffleanswers in root.iter('shuffleanswers'):
                shuffleanswers.text = elements_contenu["melange_reponse"]

            for answernumbering in root.iter('answernumbering'):
                answernumbering.text = elements_contenu["numerotation"]

            i = 0
            for answer in root.iter('answer'):
                answer.attrib["fraction"] = elements_contenu["liste_note"][i]
                for question in answer.iter('text'):
                    if question.text is not None:
                        question.text = elements_contenu["liste_reponse"][i]
                for feedback in answer.iter('feedback'):
                    for feed in feedback.iter('text'):
                        feed.text = elements_contenu["liste_feedback"][i]
                i += 1

            if i < len(elements_contenu["liste_reponse"]):
                for question in root.iter('question'):
                    # eltQuestion = root.iter("question")
                    new_answer = SubElement(question, "answer")
                    new_answer.attrib["fraction"] = elements_contenu["liste_note"][i]
                    new_answer.attrib["format"] = "html"
                    text = SubElement(new_answer, "text")
                    text.text = elements_contenu["liste_reponse"][i]
                    feedback = SubElement(new_answer, "feedback")
                    text_feedback = SubElement(feedback, "text")
                    text_feedback.text = elements_contenu["liste_feedback"][i]
                    i += 1

        elif elements_contenu["type_question"] == "numerical" or \
                elements_contenu["type_question"] == "calculatedsimple":
            pass

            for unit in root.iter('unitgradingtype'):
                unit.text = elements_contenu["traitement_unite"]

            for showunits in root.iter('showunits'):
                showunits.text = elements_contenu["saisie_unite"]

            for unitsleft in root.iter('unitsleft'):
                unitsleft.text = elements_contenu["position_unite"]

            i = 0
            for unit_name in root.iter('unit_name'):
                if unit_name is not None:
                    unit_name.text = elements_contenu["liste_unite_nom"][i]
                    i += 1

            i = 0
            for multiplier in root.iter('multiplier'):
                if multiplier is not None:
                    multiplier.text = elements_contenu["liste_unite_coef"][i]
                    i += 1

            # on ajoute les éventuelles nouvelles unités
            if i < len(elements_contenu["liste_unite_nom"]):
                for question in root.iter('question'):
                    if sum(1 for _ in question.iter('units')) == 0:
                        creer_units = SubElement(question, "units")
                        new_unit = SubElement(creer_units, "unit")
                        multiplier = SubElement(new_unit, "multiplier")
                        multiplier.text = elements_contenu["liste_unite_coef"][i]
                        unit_name = SubElement(new_unit, "unit_name")
                        unit_name.text = elements_contenu["liste_unite_nom"][i]
                    else:
                        for unit in question.iter('units'):
                            new_unit = SubElement(unit, "unit")
                            multiplier = SubElement(new_unit, "multiplier")
                            multiplier.text = elements_contenu["liste_unite_coef"][i]
                            unit_name = SubElement(new_unit, "unit_name")
                            unit_name.text = elements_contenu["liste_unite_nom"][i]

            i = 0
            for answer in root.iter('answer'):
                answer.attrib["fraction"] = elements_contenu["liste_note"][i]
                for question in answer.iter('text'):
                    if question.text is not None:
                        question.text = elements_contenu["liste_reponse"][i]
                for feedback in answer.iter('feedback'):
                    for feed in feedback.iter('text'):
                        feed.text = elements_contenu["liste_feedback"][i]
                for tolerance in answer.iter('tolerance'):
                    if tolerance.text is not None:
                        tolerance.text = elements_contenu["liste_tolerance"][i]
                if elements_contenu["type_question"] == "calculatedsimple":
                    for type in answer.iter('tolerancetype'):
                        if type.text is not None:
                            type.text = elements_contenu["liste_tolerance_type"][i]
                    for format in answer.iter('correctanswerformat'):
                        if format.text is not None:
                            format.text = elements_contenu["liste_reponse_format"][i]
                    for length in answer.iter('correctanswerlength'):
                        if length.text is not None:
                            length.text = elements_contenu["liste_reponse_taille"][i]
                i += 1

            # on ajoute les éventuelles nouvelles réponses
            if i < len(elements_contenu["liste_reponse"]):
                for question in root.iter('question'):
                    new_answer = SubElement(question, "answer")
                    new_answer.attrib["fraction"] = elements_contenu["liste_note"][i]
                    new_answer.attrib["format"] = "moodle_auto_format"
                    text = SubElement(new_answer, "text")
                    text.text = elements_contenu["liste_reponse"][i]
                    feedback = SubElement(new_answer, "feedback")
                    feedback.attrib["format"] = "html"
                    text_feedback = SubElement(feedback, "text")
                    text_feedback.text = elements_contenu["liste_feedback"][i]
                    tolerance = SubElement(new_answer, "tolerance")
                    tolerance.text = elements_contenu["liste_tolerance"][i]
                    if elements_contenu["type_question"] == "calculatedsimple":
                        tolerance_type = SubElement(new_answer, "tolerancetype")
                        tolerance_type.text = elements_contenu["liste_tolerance_type"][i]
                        correct_answer_format = SubElement(new_answer, "correctanswerformat")
                        correct_answer_format.text = elements_contenu["liste_reponse_format"][i]
                        correct_answer_length = SubElement(new_answer, "correctanswerlength")
                        correct_answer_length.text = elements_contenu["liste_reponse_taille"][i]
                    i += 1

        # on maj le fichier du contenu
        Utils().changer_texte_fichier(ObjectId(id_contenu), ET.tostring(root).decode("utf-8"),
                                      elements_contenu["nom_question"])
        return ET.tostring(root)


