from testUtils import *


def test_contenu(client):
    login = creerUtilisateurRandom(client)
    id = creerContenuRandom(client, login)

    # get contenu par id
    rv = client.get("/contenus/getContenu/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200
    # on récupère le texte
    rv = client.get("/contenus/getTexteContenu/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200
    assert "Texte du contenu" == json.loads(rv.data)
    # on créée un tiers avec Jacques
    # pprint("id : ", id["$oid"])
    loginDeux = creerUtilisateurRandom(client)
    data = {"id_tiers": json.dumps(id), "auteur": loginDeux, "version": 1}
    rv = client.patch("/contenus/ajouterTiers", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    # get Contenu Panier (chez Charles)
    rv = client.get("/contenus/getContenuPanier/" + login + "?perso=True&tiers=True&en_cours=True",
                    headers=get_headers(client))
    assert rv.status_code == 200
    assert len(json.loads(rv.data)) == 1

    loginTrois = creerUtilisateurRandom(client)

    # get Contenu Banque (chez Valery)
    rv = client.get("/contenus/getContenuBanque/" + loginTrois, headers=get_headers(client))
    assert rv.status_code == 200
    # Travaillant sur la vraie base, ce n'est pas possible de faire ce test
    # assert len(json.loads(rv.data)) == 1
    contenusBanque = len(json.loads(rv.data))

    # get All Contenus
    rv = client.get("/contenus/allContenus", headers=get_headers(client))
    assert rv.status_code == 200
    # Travaillant sur la vraie base, ce n'est pas possible de faire ce test
    assert len(json.loads(rv.data)) == contenusBanque

    # on supprime le tiers
    rv = client.delete("/contenus/deleteTiers?id_tiers=" + json.dumps(id) + "&auteur=" + loginDeux + "&note=-",
                       headers=get_headers(client))
    assert rv.status_code == 200
    # on supprime le contenu perso
    rv = client.delete("/contenus/deletePerso/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200

    # on supprime les trois utilisateurs créés
    supprimerUtilisateur(client, login)
    supprimerUtilisateur(client, loginDeux)
    supprimerUtilisateur(client, loginTrois)


def test_supprimer_contenu_en_tiers_chez_qqun_autre(client):
    # on créée un utilisateur quelconque
    login = creerUtilisateurRandom(client)
    # on lui assigne un contenu
    id = creerContenuRandom(client, login)

    # on créée un second utilisateur quelconque et on lui donne le contenu ci dessus dans ses tiers
    loginDeux = creerUtilisateurRandom(client)
    data = {"id_tiers": json.dumps(id), "auteur": loginDeux, "version": 1}
    rv = client.patch("/contenus/ajouterTiers", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    # On supprime le contenu perso de l'utilisateur 'login'
    rv = client.delete("/contenus/deletePerso/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200

    # on vérifie le panier de l'utilisateur 'loginDeux'
    rv = client.get("/contenus/getContenuPanier/" + loginDeux + "?perso=True&tiers=True&en_cours=True",
                    headers=get_headers(client))
    assert rv.status_code == 200
    assert len(json.loads(rv.data)) == 1

    # avec loginTrois, on vérifie le contenu (via la banque)
    loginTrois = creerUtilisateurRandom(client)
    # On vérifie que le contenu est tjrs dispo (chez Valery) et que son auteur est Aucun
    rv = client.get("/contenus/getContenuBanque/" + loginTrois, headers=get_headers(client))
    assert rv.status_code == 200
    # Impossible de tester le nombre de contenus en banque en travaillant sur la vraie base
    # assert len(json.loads(rv.data)) == 1
    contenus = json.loads(rv.data)
    filtered_contenu = [contenu for contenu in contenus if contenu["_id"] == id]
    assert filtered_contenu[0]["auteur"] == "Aucun"
    from modele.contenu_dao import ContenuDao
    contenu = ContenuDao().get_contenu_par_id(id)
    # login 2 modifie son tiers (désormais d'auteur Aucun)
    data = {"auteur": loginDeux, "id_contenu": json.dumps(id), "provenance": "tiers", "version": 1}
    # création d'un contenu en_cours
    rv = client.post("/en_cours/newEnCours", json=data, headers=get_headers(client))
    assert rv.status_code == 201
    id_en_cours = json.loads(rv.data)["_id"]

    # on renseigne le nouveau titre du contenu en cours
    data = {"titre": "nouveauTitreContenu", "id_contenu_en_cours": json.dumps(id_en_cours)}
    rv = client.patch("/en_cours/changerTitre", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    # on prend le contenu en cours créé juste au dessus pour mettre à jour le tiers anonyme
    data = {"auteur": loginDeux, "id_contenu_en_cours": json.dumps(id_en_cours)}
    rv = client.patch("/contenus/majContenu", json=data, headers=get_headers(client))
    assert rv.status_code == 200

    # On vérifie que le contenu est tjrs dispo (chez Valery) et que son auteur est Jacques
    rv = client.get("/contenus/getContenuBanque/" + loginTrois, headers=get_headers(client))
    assert rv.status_code == 200
    # travail sur la vraie base = tests sur la banque impossibles
    # assert len(json.loads(rv.data)) == 1
    contenus = json.loads(rv.data)
    filtered_contenu = [contenu for contenu in contenus if contenu["_id"] == id]
    assert filtered_contenu[0]["auteur"] == loginDeux

    rv = client.delete("/contenus/deletePerso/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200

    # on supprime les trois utilisateurs créés
    supprimerUtilisateur(client, login)
    supprimerUtilisateur(client, loginDeux)
    supprimerUtilisateur(client, loginTrois)


def test_notation_contenu(client):
    # on créée un utilisateur quelconque
    login = creerUtilisateurRandom(client)
    # on lui assigne un contenu
    id = creerContenuRandom(client, login)
    # on créée un second utilisateur quelconque et on lui donne le contenu ci dessus dans ses tiers
    loginDeux = creerUtilisateurRandom(client)
    data = {"id_tiers": json.dumps(id), "auteur": loginDeux, "version": 1}
    rv = client.patch("/contenus/ajouterTiers", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    # login2 note le contenu
    data = {"auteur": loginDeux, "id_contenu": json.dumps(id), "version": 1, "note": 4}
    rv = client.patch("/contenus/noterContenu", json=data, headers=get_headers(client))
    assert rv.status_code == 200

    # pour essai, on récupère la note juste mise
    rv = client.get("/contenus/recupererNote?id_contenu=" + json.dumps(id) + "&version=1&auteur=" + loginDeux,
                    headers=get_headers(client))
    assert rv.status_code == 200
    # la note ressort en caracteres et non en entier
    assert json.loads(rv.data) == '4'

    # on teste la note globale du contenu
    rv = client.get("/contenus/recupererNoteGlobale?id_contenu=" + json.dumps(id) + "&version=1",
                    headers=get_headers(client))
    assert rv.status_code == 200
    print("popapopapa : ", json.loads(rv.data))
    assert json.loads(rv.data) == '4.0'

    # un troisieme utilisateur note
    # login2 note le contenu
    loginTrois = creerUtilisateurRandom(client)
    data = {"auteur": loginTrois, "id_contenu": json.dumps(id), "version": 1, "note": 3}
    rv = client.patch("/contenus/noterContenu", json=data, headers=get_headers(client))
    assert rv.status_code == 200

    # pour essai, on récupère la note juste mise
    rv = client.get("/contenus/recupererNote?id_contenu=" + json.dumps(id) + "&version=1&auteur=" + loginTrois,
                    headers=get_headers(client))
    assert rv.status_code == 200
    # la note ressort en caracteres et non en entier
    assert json.loads(rv.data) == '3'

    # on teste la note globale du contenu
    rv = client.get("/contenus/recupererNoteGlobale?id_contenu=" + json.dumps(id) + "&version=1",
                    headers=get_headers(client))
    assert rv.status_code == 200
    print("popapopapa : ", json.loads(rv.data))
    assert json.loads(rv.data) == '3.5'

    # on supprime le tiers & le contenu
    rv = client.delete("/contenus/deleteTiers?id_tiers=" + json.dumps(id) + "&auteur=" + loginDeux + "&note=4",
                       headers=get_headers(client))
    assert rv.status_code == 200

    # on supprime le tiers & le contenu
    rv = client.delete("/contenus/deleteTiers?id_tiers=" + json.dumps(id) + "&auteur=" + loginTrois + "&note=3",
                       headers=get_headers(client))
    assert rv.status_code == 200

    rv = client.delete("/contenus/deletePerso/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200
    # et les trois utilisateurs

    supprimerUtilisateur(client, login)
    supprimerUtilisateur(client, loginDeux)
    supprimerUtilisateur(client, loginTrois)


## Deux fonctions de tags, une pour les tags fixes, une pour les tags libres
def test_tags_fixes_contenu(client):
    # on créée un utilisateur quelconque
    login = creerUtilisateurRandom(client)
    # on lui assigne un contenu
    id = creerContenuRandom(client, login)

    # pour essai, on récupère le tag de base
    rv = client.get("/contenus/getTags/" + json.dumps(id),
                    headers=get_headers(client))
    assert rv.status_code == 200
    # print(json.loads(rv.data))
    assert json.loads(rv.data) == ["tag1"]

    # on modifie le tag de base
    data = {"id_contenu": json.dumps(id), "tags": ["tag2"]}
    rv = client.patch("/contenus/modifierTags", json=data,
                      headers=get_headers(client))
    assert rv.status_code == 200

    # on récupère les nouveaux tags
    rv = client.get("/contenus/getTags/" + json.dumps(id),
                    headers=get_headers(client))
    assert rv.status_code == 200
    # print(json.loads(rv.data))
    assert json.loads(rv.data) == ["tag2"]

    rv = client.delete("/contenus/deletePerso/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200

    supprimerUtilisateur(client, login)


## Deux fonctions de tags, une pour les tags fixes, une pour les tags libres
def test_tags_libres_contenu(client):
    # on créée un utilisateur quelconque
    login = creerUtilisateurRandom(client)
    # on lui assigne un contenu
    id = creerContenuRandom(client, login)

    # pour essai, on récupère les tags libres du contenu et de l'auteur (censés être vides tous les deux)
    rv = client.get("/contenus/getTagLibre/" + login,
                    headers=get_headers(client))
    assert rv.status_code == 200
    # print(json.loads(rv.data))
    assert json.loads(rv.data) == []

    rv = client.get("/contenus/getTagLibreContenu/" + json.dumps(id) + "/" + login,
                    headers=get_headers(client))
    assert rv.status_code == 200
    # print(json.loads(rv.data))
    assert json.loads(rv.data) == []

    # on créée un 2e utilisateur qui s'approprie id, va créer un tag libre et l'affecter à id
    loginDeux = creerUtilisateurRandom(client)
    data = {"id_tiers": json.dumps(id), "auteur": loginDeux, "version": 1}
    rv = client.patch("/contenus/ajouterTiers", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    data = {"auteur": loginDeux, "tag": "tagUn"}
    rv = client.patch("/contenus/newTagLibre", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    # on vérifie que le tag soit bien "propriété" de loginDeux
    rv = client.get("/contenus/getTagLibre/" + loginDeux, headers=get_headers(client))
    assert rv.status_code == 200
    assert json.loads(rv.data) == ["tagUn"]

    # on associe le tag au contenu id
    data = {"auteur": loginDeux, "tag": "tagUn", "id_contenu": json.dumps(id)}
    rv = client.patch("/contenus/associerTagLibre", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    # on vérifie les tags que login2 a associé à son tiers id
    rv = client.get("/contenus/getTagLibreContenu/" + json.dumps(id) + "/" + loginDeux, headers=get_headers(client))
    assert rv.status_code == 200
    assert json.loads(rv.data) == ["tagUn"]
    print(json.loads(rv.data))

    # on dissocie le tag du contenu
    data = {"auteur": loginDeux, "tag": "tagUn", "id_contenu": json.dumps(id)}
    rv = client.patch("/contenus/dissocierTagLibre", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    # on vérifie les tags que login2 n'a plus de tag associé
    rv = client.get("/contenus/getTagLibreContenu/" + json.dumps(id) + "/" + loginDeux, headers=get_headers(client))
    assert rv.status_code == 200
    assert json.loads(rv.data) == []
    print(json.loads(rv.data))

    rv = client.delete("/contenus/deleteTiers?id_tiers=" + json.dumps(id) + "&auteur=" + loginDeux + "&note=-",
                       headers=get_headers(client))
    assert rv.status_code == 200

    rv = client.delete("/contenus/deletePerso/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200

    supprimerUtilisateur(client, login)
    supprimerUtilisateur(client, loginDeux)
